/*
 * Lehrstuhl Softwaretechnologie, TU Dresden, Datei Bibliothek.java
 *
 * Bibliothek-Klasse implementiert eine einfache Bibliothek, die Bcher sortiert
 * in ein Set aufnehmen, auf Existenz von Bchern prfen und den Bestand nach
 * Autoren auflisten kann  (Variante mit Generics)
 */

import java.util.Collection;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;

public class Bibliothek {

    private Set<Buch> bestand;

    public Bibliothek() {
        bestand = new TreeSet<Buch>();
    }

    public boolean sortiertesEinfuegen(Buch neuesBuch) {
        return bestand.add(neuesBuch);
    }

    public Buch sucheNachISBN(String isbn) {
        for (Buch b : bestand) {
            if (isbn.equals(b.getIsbn())) {
                return b;
            }
        }
        return null;
    }

    public Collection<Buch> sucheNachAutor(String autor) {
        // enthlt spter alle Ergebnisse
        Collection<Buch> suchergebnis = new HashSet<Buch>();
        for (Buch b : bestand) {
            if (autor.equals(b.getAutor())) {
                // ein Buch des Autors wurde gefunden
                suchergebnis.add(b);
            }
        }

        return suchergebnis;
    }

    public Map<String,Set<Buch>> bestandNachAutorAuflisten() {
        Map<String,Set<Buch>> ergebnis = new TreeMap<String,Set<Buch>>();
        Set<Buch> buecherEinesAutors;
        for (Buch b: bestand) {
            // Set von Bchern wird mittels Schlssel "Autor" aus der Map geholt
            buecherEinesAutors = ergebnis.get(b.getAutor());
            if (buecherEinesAutors == null) {
                // noch kein Set dieses Autors vorhanden, neues anlegen
                buecherEinesAutors = new TreeSet<Buch>();
                // in die Map eintragen
                ergebnis.put(b.getAutor(), buecherEinesAutors);
            }
            // das Set existiert nun definitiv
            buecherEinesAutors.add(b);
        }
        return ergebnis;
    }

    public Collection<Buch> getBestand() {
        return bestand;
    }
}