//package de.tud.inf.st.graphs;

import java.util.List;
import org.jgrapht.GraphPath;
import org.jgrapht.Graphs;
import org.jgrapht.UndirectedGraph;
import org.jgrapht.alg.DijkstraShortestPath;
import org.jgrapht.graph.DefaultEdge;
import org.jgrapht.graph.SimpleGraph;

/**
 * This class demonstrates the Dijkstra algorithm that can be performed on
 * directed graphs. After constructing a basic directed graph, it computes
 *
 * @author Birgit Demuth
 * 2012-04-23
 */

public class DijkstraDemo {

	private static UndirectedGraph<String, DefaultEdge> createCampusDVBNet() {
		UndirectedGraph<String, DefaultEdge>  campusDVBNet =
			new SimpleGraph<String, DefaultEdge>(DefaultEdge.class);

		// add vertexes (Stop objects)
		campusDVBNet.addVertex("Nthnitzer Str");
		campusDVBNet.addVertex("Regensburger Str");
		campusDVBNet.addVertex("Helmholtzstr");
		campusDVBNet.addVertex("Stadtgutstr");
		campusDVBNet.addVertex("Rcknitzhhe");
		campusDVBNet.addVertex("Zellescher Weg");
		campusDVBNet.addVertex("Mnchner Platz");
		campusDVBNet.addVertex("SLUB");
		campusDVBNet.addVertex("Strehlener Platz");
		campusDVBNet.addVertex("Nrnberger Platz");
		campusDVBNet.addVertex("Technische Universitt");
		campusDVBNet.addVertex("Uhlandstr");
		campusDVBNet.addVertex("Lenneplatz");
		campusDVBNet.addVertex("Reichenbachstr");
		campusDVBNet.addVertex("Hauptbahnhof");
		campusDVBNet.addVertex("Hauptbahnhof Nord");
		campusDVBNet.addVertex("Gret-Palucca-Str");

		// add edges (direct links between two stops)
		campusDVBNet.addEdge("Nthnitzer Str", "Regensburger Str");
		campusDVBNet.addEdge("Regensburger Str", "Helmholtzstr");
		campusDVBNet.addEdge("Helmholtzstr", "Stadtgutstr");
		campusDVBNet.addEdge("Stadtgutstr", "Rcknitzhhe");
		campusDVBNet.addEdge("Rcknitzhhe", "Zellescher Weg");
		campusDVBNet.addEdge("Zellescher Weg", "SLUB");
		campusDVBNet.addEdge("Zellescher Weg", "Strehlener Platz");
		campusDVBNet.addEdge("Strehlener Platz", "Uhlandstr");
		campusDVBNet.addEdge("Strehlener Platz", "Lenneplatz");
		campusDVBNet.addEdge("Lenneplatz", "Gret-Palucca-Str");
		campusDVBNet.addEdge("Gret-Palucca-Str", "Hauptbahnhof Nord");
		campusDVBNet.addEdge("Gret-Palucca-Str", "Uhlandstr");
		campusDVBNet.addEdge("Gret-Palucca-Str", "Hauptbahnhof");
		campusDVBNet.addEdge("Hauptbahnhof Nord", "Hauptbahnhof");
		campusDVBNet.addEdge("Hauptbahnhof", "Reichenbachstr");
		campusDVBNet.addEdge("Reichenbachstr", "Technische Universitt");
		campusDVBNet.addEdge("Reichenbachstr", "Nrnberger Platz");
		campusDVBNet.addEdge("Nrnberger Platz", "Technische Universitt");
		campusDVBNet.addEdge("Nrnberger Platz", "Mnchner Platz");
		campusDVBNet.addEdge("Mnchner Platz", "Nthnitzer Str");
		campusDVBNet.addEdge("Technische Universitt", "SLUB");

		return campusDVBNet;
	}

	public static void main(String args[]) {

		// constructs a directed graph with the specified vertices and edges
		UndirectedGraph<String, DefaultEdge> campusDVBNet = createCampusDVBNet();
		System.out.println("CampusDVBNet: " + campusDVBNet.toString() + "\n");

		// (c) Shortest path with Dijkstra's method
		DijkstraShortestPath<String, DefaultEdge> shortestPath = new DijkstraShortestPath<String, DefaultEdge>(
				campusDVBNet,"Helmholtzstr" ,"Hauptbahnhof" );
		System.out.println("\nshortest path between Helmholtzstr. and Hauptbahnhof ("
				+ shortestPath.getPathLength() + "):");
		GraphPath<String, DefaultEdge> path = shortestPath.getPath();
		// Graphs is an algorithm class (helper class)
		List<String> nodeList = Graphs.getPathVertexList(path);
		for (String node : nodeList) {
			System.out.println("node: " + node);
		}
	}
}